import * as d3 from 'd3';
import { globalSetting } from '../global';

export class ConnectionRender {
    /**
     * 
     * @param {d3.Selection} selection 
     * @param {BasicD3DataItem} connections
     */
    constructor(selection, connections) {
        this.parentSelection = selection;
        this.data = connections;
    }

    render() {
        const data = this.data;

        let connectionSelections = this.parentSelection.selectAll('circle')
            .data(data)
            .enter()
            .append('circle')
            .attr('cx', data => data.x)
            .attr('cy', data => data.y)
            .attr('width', data => data.width)
            .attr('height', data => data.height)
        
        if (globalSetting.renderAnimation) {
            connectionSelections = connectionSelections
                .transition()
                .duration(1000);
        }

        connectionSelections
            .attr('fill', d => d.fill)
            .attr('r', d => d.r)
            .on('end', function (data) {
                const connectionSelection = d3.select(this);
                registerDragEvent(connectionSelection, data);
            });

        this.selections = connectionSelections;
        return connectionSelections;
    }
}

/**
 * @description 注册关于 器件 的拖动事件
 * 
 * 需要提取最小拓扑子图，然后重新调整各个区域的尺寸
 * @param {d3.Selection} selection
 * @param {any} data
 */
export function registerDragEvent(selection, data) {
    // 创建拖拽行为
    const drag = d3.drag();
    drag.on("drag", event => dragged(event, selection, data));
    selection.call(drag);
}


/**
 * 
 * @param {d3.D3DragEvent} event 
 * @param {d3.Selection} selection 
 */
function dragged(event, selection, data) {
    data.x = event.x;
    data.y = event.y;
    selection.attr('x', event.x).attr('y', event.y);
}