import * as d3 from 'd3';
import ELK from 'elkjs';
import { globalLookup, globalSetting } from '../global';

export class PortRender {
    /**
     * 
     * @param {d3.Selection} selection 
     * @param {BasicD3DataItem} ports
     */
    constructor(selection, ports) {
        this.parentSelection = selection;
        this.data = ports;
    }

    render() {
        const data = this.data;

        let portSelections = this.parentSelection.selectAll('rect')
            .data(data)
            .enter()
            .append('rect')
            .attr('x', data => data.x)
            .attr('y', data => data.y)
            .attr('width', data => data.width)
            .attr('height', data => data.height)
            .attr('fill', d => d.fill);
        
        portSelections.append("text")
            .attr("x", d => d.width / 2) // 文本的 x 坐标（居中）
            .attr("y", d => d.height / 2) // 文本的 y 坐标（居中）
            .attr("text-anchor", "middle") // 文本水平居中
            .attr("dominant-baseline", "middle") // 文本垂直居中
            .text(d => d.text)      // 设置文本内容
            .attr("fill", "black"); // 文本颜色

        if (globalSetting.renderAnimation) {
            portSelections = portSelections
                .transition()
                .duration(1000)
        }
        portSelections
            .attr('stroke', 'var(--main-color)')
            .attr('stroke-width', 2)
            .attr('rx', d => d.rx)
            .attr('ry', d => d.ry)
            .on('end', function (data) {
                const portSelection = d3.select(this);
                registerDragEvent(portSelection, data);
            });
        
        this.selections = portSelections;
        return portSelections;
    }
}

/**
 * @description 注册关于 器件 的拖动事件
 * 
 * 需要提取最小拓扑子图，然后重新调整各个区域的尺寸
 * @param {d3.Selection} selection
 * @param {any} data
 */
export function registerDragEvent(selection, data) {
    // 创建拖拽行为
    const drag = d3.drag();
    drag.on("drag", event => dragged(event, selection, data));
    selection.call(drag);
}


/**
 * 
 * @param {d3.D3DragEvent} event 
 * @param {d3.Selection} selection 
 */
function dragged(event, selection, data) {
    data.x = event.x;
    data.y = event.y;
    selection.attr('x', event.x).attr('y', event.y);
}